@@ -15,15 +15,16 @@ Reporting functions for codeaudit
 
 import re
 import os
+from pathlib import Path
 
 import pandas as pd
 import datetime
 
 from codeaudit.security_checks import perform_validations , ast_security_checks
-from codeaudit.filehelpfunctions import get_filename_from_path , collect_python_source_files , read_in_source_file 
+from codeaudit.filehelpfunctions import get_filename_from_path , collect_python_source_files , read_in_source_file , has_python_files , is_ast_parsable
 from codeaudit.altairplots import multi_bar_chart
 from codeaudit.totals import get_statistics , overview_count , overview_per_file , total_modules
-from codeaudit.checkmodules import get_imported_modules , check_module_on_vuln , get_all_modules , get_imported_modules_by_file
+from codeaudit.checkmodules import get_imported_modules , check_module_vulnerability , get_all_modules , get_imported_modules_by_file
 from codeaudit.htmlhelpfunctions import dict_to_html , json_to_html , dict_list_to_html_table
 from codeaudit import __version__
 
@@ -51,13 +52,17 @@ def overview_report(directory, filename=DEFAULT_OUTPUT_FILE):
         print(f"ERROR: '{directory}' is not a directory (maybe you try to run it for a single file)")
         print(f"This function only works for directories which contains one or more Python source code files (*.py). ")
         exit(1)
+    #Check if the directory has Python files
+    if not has_python_files(directory):
+        print(f'Error: Directory path {directory} contains no Python files.')
+        exit(1)
     result = get_statistics(directory)
     modules = total_modules(directory)    
     df = pd.DataFrame(result)
     df['Std-Modules'] = modules['Std-Modules']
     df['External-Modules'] = modules['External-Modules']
     overview_df = overview_count(df)
-    html = '<h1>' + f'Codeaudit overview report' + '</h1><br>'
+    html = '<h1>' + f'Python Code Audit overview report' + '</h1><br>'
     html += f'<p>Codeaudit overview scan of the directory:<b> {directory}</b></p>' 
     html += f'<h2>Summary</h2>'
     html += overview_df.to_html(escape=True,index=False)
@@ -95,15 +100,16 @@ def overview_report(directory, filename=DEFAULT_OUTPUT_FILE):
     html += '<h2>Visual Overview</h2>'    
     html += extract_altair_html(plot_html)    
     create_htmlfile(html,filename)
-    
+        
+        
 
-def file_scan_report(file_to_scan , filename=DEFAULT_OUTPUT_FILE):
-    """Reports potential security issues for a single Python file.
-    
-    This function performs security validations on the specified file, 
+def scan_report(input_path , filename=DEFAULT_OUTPUT_FILE):
+    """Scans Python files or directories(packages) for vulnerabilities and reports potential issues.
+        
+    This function performs security validations on the specified file or directory, 
     formats the results into an HTML report, and writes the output to an HTML file. 
     
-    You can specify the name and directory for the generated HTML report.
+    You can specify the name of the outputfile and directory for the generated HTML report. Make sure you chose the extension `.html` since the output file is a static html file.
 
     Parameters:
         file_to_scan (str)      : The full path to the Python source file to be scanned.
@@ -113,16 +119,27 @@ def file_scan_report(file_to_scan , filename=DEFAULT_OUTPUT_FILE):
     Returns:
         None - A HTML report is written as output
     """
-    scan_output = perform_validations(file_to_scan)    
-    file_report_html = single_file_report(file_to_scan , scan_output)    
-    name_of_file = get_filename_from_path(file_to_scan)
-    html = '<h1>Codeaudit report</h1>' #prepared to be embedded to display multiple reports, so <h2> used
-    html += f'<h2>Result of scan of file {name_of_file}</h2>'    
-    html += '<p>' + f'Location of the file: {file_to_scan} </p>'  
-    html += file_report_html    
-    html += '<br>'
-    html += DISCLAIMER_TEXT
-    create_htmlfile(html,filename)  
+      # Check if the input is a valid directory or a single valid Python file
+    file_path = Path(input_path)
+    if file_path.is_dir():
+        directory_scan_report(input_path , filename ) #create a package aka directory scan report
+    elif file_path.suffix == ".py" and file_path.is_file() and is_ast_parsable(input_path):        
+        #create a sast file check report
+        scan_output = perform_validations(input_path)
+        file_report_html = single_file_report(input_path , scan_output)    
+        name_of_file = get_filename_from_path(input_path)
+        html = '<h1>Python Code Audit Report</h1>' #prepared to be embedded to display multiple reports, so <h2> used
+        html += f'<h2>Result of scan of file {name_of_file}</h2>'    
+        html += '<p>' + f'Location of the file: {input_path} </p>'  
+        html += file_report_html    
+        html += '<br>'
+        html += DISCLAIMER_TEXT
+        create_htmlfile(html,filename)
+    else:
+        #File is NOT a valid Python file, can not be parsed or directory is invalid.
+        print(f"Error: '{input_path}' isn't a valid Python file or directory path.")
+        
+   
 
 def single_file_report(filename , scan_output):
     """Function to DRY for a codescan when used for single for CLI or within a directory scan"""
@@ -168,7 +185,7 @@ def single_file_report(filename , scan_output):
     html += '<summary>Click to see details for used modules in this file.</summary>' 
     modules_found = get_imported_modules_by_file(filename)
     html += dict_to_html(modules_found)
-    html += f'<p><i>Use the command:<br><b><code>codeaudit modulescan {filename}</code></b><br> to check if vulnerabilities are reported in an external module used by this file.</i></p>' 
+    html += f'<p>To check for <b>reported vulnerabilities</b> in external modules used by this file, use the command:<br><div class="code-box">codeaudit modulescan {filename}</div><br></p>'     
     html += '</details>'           
     return html 
 
@@ -195,12 +212,17 @@ def directory_scan_report(directory_to_scan , filename=DEFAULT_OUTPUT_FILE):
         exit(1) 
 
     collection_ok_files = [] # create a collection of files with no issues found    
-    html = '<h1>Codeaudit report</h1>'     
+    html = '<h1>Python Code Audit Report</h1>'     
     files_to_check = collect_python_source_files(directory_to_scan)
-    html += '<h2>Directory scan report</h2>'     
-    html += f'<p>Below the result of the Codeaudit scan of the directory:<b> {directory_to_scan}</b></p>' 
+    html += '<h2>Directory scan report</h2>'         
+    name_of_package = get_filename_from_path(directory_to_scan)
+    html += f'<p>Below the result of the Codeaudit scan of the package or directory:<b> {name_of_package}</b></p>' 
     html += f'<p>Total Python files found: {len(files_to_check)}</p>'
-    for file_to_scan in files_to_check:
+    number_of_files = len(files_to_check)
+    print(f'Number of files that are checked for security issues:{number_of_files}')
+    printProgressBar(0, number_of_files, prefix='Progress:', suffix='Complete', length=50)    
+    for i,file_to_scan in enumerate(files_to_check):
+        printProgressBar(i + 1, number_of_files, prefix='Progress:', suffix='Complete', length=50)
         scan_output = perform_validations(file_to_scan)
         data = scan_output["result"]
         if data:
@@ -214,7 +236,7 @@ def directory_scan_report(directory_to_scan , filename=DEFAULT_OUTPUT_FILE):
             collection_ok_files.append({'filename' : file_name_with_no_issue ,
                                         'directory': file_to_scan})
     html += '<h2>Files in directory with no security issues</h2>'
-    html += f'<p>Total Python files with no detected security issue: {len(collection_ok_files)}</p>'
+    html += f'<p>Total Python files <b>without</b> detected security issues: {len(collection_ok_files)}</p>'
     html += '<p>The Python files with no security issues <b>detected</b> by codeaudit are:<p>'        
     html += dict_list_to_html_table(collection_ok_files)     
     html += '<br>'
@@ -224,14 +246,14 @@ def directory_scan_report(directory_to_scan , filename=DEFAULT_OUTPUT_FILE):
 
 
 def report_module_information(inputfile,reportname=DEFAULT_OUTPUT_FILE):
-    """Reports module information per file.""" 
+    """Reports module vulnerability information.""" 
     source = read_in_source_file(inputfile)
     used_modules = get_imported_modules(source)
     # Initial call to print 0% progress
     external_modules = used_modules['imported_modules']    
     l = len(external_modules)
     printProgressBar(0, l, prefix='Progress:', suffix='Complete', length=50)    
-    html = '<h1>Codeaudit Report</h1>'
+    html = '<h1>Python Code Audit Report</h1>'
     html += f'<h2>Module information for file {inputfile}</h2>'    
     html += dict_to_html(used_modules)    
     #Now vuln info per external module        
@@ -239,7 +261,7 @@ def report_module_information(inputfile,reportname=DEFAULT_OUTPUT_FILE):
         html += '<h2>Vulnerability information for detected modules</h2>'    
     for i,module in enumerate(external_modules):  #sorted for nicer report
         printProgressBar(i + 1, l, prefix='Progress:', suffix='Complete', length=50)
-        vuln_info = check_module_on_vuln(module)
+        vuln_info = check_module_vulnerability(module)
         if not vuln_info:
             html += f'<h3>Vulnerability information for module <b>{module}</b></h3> '
             html += f'<li>No information found in OSV Database for module: <b>{module}</b>.</li> '
@@ -274,7 +296,7 @@ def create_htmlfile(html_input,outputfile):
     output += f'<style>\n{css_content}\n</style>'    
     output += '<script src="https://cdn.jsdelivr.net/npm/vega@5"></script>' # needed for altair plots
     output += '<script src="https://cdn.jsdelivr.net/npm/vega-lite@5"></script>' # needed for altair plots
-    output += '<script src="https://cdn.jsdelivr.net/npm/vega-embed@6"></script>' # needed for altair plots
+    output += '<script src="https://cdn.jsdelivr.net/npm/vega-embed@6"></script>' # needed for altair plots   
     output += '</head><body>'
     output += '<div class="container">'
     output += html_input
@@ -284,7 +306,7 @@ def create_htmlfile(html_input,outputfile):
     output += '<footer>'
     output += '<hr>'
     output += f'<p><small>This security report is created on: {timestamp_str}, with <a href="https://github.com/nocomplexity/codeaudit">codeaudit</a> version {code_audit_version} </small></p>'
-    output += '<p><small>Check the <a href="https://nocomplexity.com/documents/codeaudit/intro.html" target="_blank">documentation</a> for help on found issues. <a href="https://github.com/nocomplexity/codeaudit">Codeaudit</a> is made with &#10084; by cyber security professionals who advocate for <a href="https://simplifysecurity.nocomplexity.com" target="_blank">simpler cyber solutions</a> that are transparent and just work. Help to make this FOSS software better!</small></p>'    
+    output += '<p><small>Check the <a href="https://nocomplexity.com/documents/codeaudit/intro.html" target="_blank">documentation</a> for help on found issues. <a href="https://github.com/nocomplexity/codeaudit">Codeaudit</a> is made with &#10084; by cyber security professionals who advocate for <a href="https://simplifysecurity.nocomplexity.com" target="_blank">open simple cyber security solutions</a>. Join the community and <a href="https://nocomplexity.com/documents/codeaudit/CONTRIBUTE.html" target="_blank">contribute </a> to make this Python Security Code Audit tool better!</small></p>'    
     output += '</footer>' 
     output += '</div>' #base container
     output += '</body></html>'
@@ -302,7 +324,7 @@ def create_htmlfile(html_input,outputfile):
         file_url = f'file://{directory_for_output}/{filename_only}'
     # Print the result
     print("\n=====================================================================")
-    print(f'Codeaudit report file created!\nPaste the line below directly into your browser bar:\n\t{file_url}\n')
+    print(f'Code Audit report file created!\nPaste the line below directly into your browser bar:\n\t{file_url}\n')
     print("=====================================================================\n")
 
 
@@ -368,7 +390,7 @@ def get_info_on_test(error):
 
 def report_implemented_tests(filename=DEFAULT_OUTPUT_FILE):
     """
-    Generate an HTML report of all implemented codeaudit security checks.
+    Creates an HTML report of all implemented security checks.
 
     This report provides a user-friendly overview of the static security checks 
     currently supported by codeaudit. It is intended to make it easier to review 
@@ -389,7 +411,7 @@ def report_implemented_tests(filename=DEFAULT_OUTPUT_FILE):
     df_checks = ast_security_checks()
     df_checks['construct'] = df_checks['construct'].apply(replace_second_dot) #Make the validation column smaller - this is the simplest way! without using styling options from Pandas!
     df_checks_sorted = df_checks.sort_values(by='construct')
-    html = '<h1>Codeaudit Implemented validations</h1>' #prepared to be embedded to display multiple reports, so <h2> used
+    html = '<h1>Python Code Audit Implemented validations</h1>' #prepared to be embedded to display multiple reports, so <h2> used
     number_of_test = len(df_checks)
     
     html += df_checks_sorted.to_html(escape=False,index=False)   
